#include <stdio.h>
#include <math.h>
#include <fftw.h>
#include <assert.h>
#include "fourier.h"

/* slow */

void
dft (frame s, spectrum S)
{
  int m;

  assert (s && S);

  for (m=0; m<N; m++)
    {
      int n;

      complex c = { 0, 0 };

      for (n=0; n<N; n++)
	{
	  real phi = -2*M_PI/N*n*m;

	  real exp_re = cos (phi);
	  real exp_im = sin (phi);

	  c_re(c) += s[n] * exp_re;
	  c_im(c) += s[n] * exp_im;
	}

      S[m] = c;
    }
}

/* fast */

static fftw_plan plan = NULL, plan_inverse = NULL;

void
fft_init (void)
{
  plan = fftw_create_plan (N, FFTW_FORWARD, FFTW_ESTIMATE | FFTW_IN_PLACE);

  plan_inverse = fftw_create_plan (N, FFTW_BACKWARD, FFTW_ESTIMATE | FFTW_IN_PLACE);
}

void
fft_exit (void)
{
  fftw_destroy_plan (plan);

  fftw_destroy_plan (plan_inverse);
}

void
fft (frame s, spectrum S)
{
  int i;

  for (i=0; i<N; i++)
    {
      c_re(S[i]) = s[i];
      c_im(S[i]) = 0;
    }

  fftw_one (plan, S, NULL);
}

void
ifft (spectrum S, frame s)
{
  int i;

  /* de-normalization */

  for (i=1; i<N; i++)
    {
      c_re(S[i]) /= 2;
      c_im(S[i]) /= 2;
    }

  fftw_one (plan_inverse, S, NULL);

  for (i=0; i<N; i++)
    {
      s[i] = c_re(S[i]);
    }
}

/* misc */

void
amp_normalize (frame amp, real sum)
{
  int i;

  amp[0] /= N;

  for (i=1; i<N; i++)
    {
      amp[i] = amp[i] * 2 / sum; 
    }
}

/* End Of File */
