#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <assert.h>
#include "file.h"
#include "fourier.h"
#include "plot.h"
#include "types.h"
#include "sound.h"
#include "complex.h"
#include "window.h"

int
main (int argc, char *argv[])
{
  FILE *input, *output;

  /* temporal */

  frame s;

  /* spectral */

  spectrum S;

  real amp[N], phs[N];

  /* benchmark */

  real bench_time = 0;
  real bench_count = 0;

  assert (sizeof(short) == 2);

  if (argc != 2)
    {
      fprintf (stderr, "usage: %s <soundfile>\n", argv[0]);
      exit (EXIT_FAILURE);
    }

  /* init */

  plot_init ();
  fft_init ();

  input = sound_file_open_read (argv[1]);
  output = sound_file_open_write ();

  /* process */

  while (sound_file_read (input, s))
    // && (getchar() != 'q'))
    {
      int i;

      real sum;

      clock_t t1, t2;

      plot_temporal (s);

      /* Fourier analysis */

      sum = hann_window (s);
      zero_phase (s);

      t1 = clock ();

#ifdef SLOW_DFT
      dft (s, S);
#else /* FFT */
      fft (s, S);
#endif
      t2 = clock ();

      bench_time += (t2 - t1);
      bench_count += 1;

      cartesian_to_polar (S, amp, phs);
      amp_normalize (amp, N);
      plot_spectral (amp);

      /*
	sleep (1);
      */

      /* transform: robotize! */

      for (i=0; i<N; i++)
	{
	  phs[i] = 0;
	}

      /* Fourier synthesis */

      polar_to_cartesian (amp, phs, S);

      ifft (S, s);

      zero_phase (s);

      /*
	plot_temporal (s);
      */

      sound_file_write (s, output);
    }

  /* exit */

  sound_file_close_write (output);
  sound_file_close_read (input);

  fft_exit ();
  plot_exit ();

  printf ("Fourier: %f s\n", bench_time / (bench_count * CLOCKS_PER_SEC));

  exit (EXIT_SUCCESS);

  return 0;
}

/* End Of File */
